/*****************************************************
Chip type           : ATmega8
*****************************************************/

#include <mega8.h>
#include <delay.h>
#include <math.h>
// Alphanumeric LCD Module functions
#asm
   .equ __lcd_port=0x12 ;PORTD
#endasm
#include <lcd.h>

// Declare your global variables here  
#define light_on  PORTB.0=1;
#define light_off PORTB.0=0;

unsigned char Ok=27, Down=29, Up=30, key=31;

const float P2=6.283185308;
const float DR=0.017453293;
const float K1=0.262516090;

const unsigned char days_month[12]={31,28,31,30,31,30,31,31,30,31,30,31};
 
unsigned char  hr, mins, sec, day=8, B5=69, C0, H=3, lat=33, cor=30; //

long int J3, Y=2009, M=3;

long int MM[2], HH[2];

float aA[2], aD[2];

float D, T, T0, TT, A, B, A0, D0, D1, E, F, J, S, L, G, V, U, V0, V1, V2; 

bit M8, W8, st;

flash char info[]="AstroTimer v.1.0";
flash char msg0[]="                ";
flash char msg1[]="Latitude";
flash char msg2[]="Longitude";
flash char msg3[]="Zone";
flash char msg4[]="Corr +- mins";
flash char msg5[]="Month";
flash char msg6[]="Day";
flash char msg7[]="Year";
flash char msg8[]="Hour";
flash char msg9[]="Min";

char delim=':';

// Declare your global variables here

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
// Place your code here
  key=PINC & 0b00011111;//  
}

// Timer 1 output compare A interrupt service routine
interrupt [TIM1_COMPA] void timer1_compa_isr(void)
{
// Place your code here
 sec++;
 TCNT1H=0;
 TCNT1L=0; 
}




void me_delay(void)
{
  while (key!=31) {delay_ms(200);};
  key=0;
}

void cld_cls(void)
{
  lcd_gotoxy(0,0);
  lcd_putsf(msg0);//16 
  HH[0]=0; MM[0]= 0; //
  HH[1]=0; MM[1]= 0; //
}

// Declare your global variables here

float Abs(float x)
{
  return(x<0?-x:x);
}


int Sgn(float x)
{  
  return(x>=0?1:-1);
}

long int Trunc (float x)
{
  if (x>=0) {x=floor(x);}
  else {x=ceil(x);}
  return (long int)x;
}

void sun_info()          
{ int i;
  for (i=0; i<2; i++)
  {
    lcd_gotoxy(6*i,0);
    lcd_putchar (HH[i]/10%10+0x30);
    lcd_putchar (HH[i]%10+0x30);
    lcd_putchar (':');
    lcd_putchar (MM[i]/10%10+0x30);
    lcd_putchar (MM[i]%10+0x30);
  }
}

void Calendar(void)
{
  D1 = Trunc(D);
  F = D - D1 - 0.5; 
  J = -Trunc(7 * (Trunc((M + 9) / 12) + Y) / 4);                                      
  S = Sgn(M - 9);
  J3 = Trunc(Y + S * Trunc(Abs(M - 9)/7)); //A / 7));
  J3 =( -(((J3 / 100) + 1) * 3 / 4));
  J = J + Trunc(275 * M / 9) + D1 + J3;
  J = J + 1721029 + 367 * Y;
  if (F >= 0) {return;};
  F = F + 1;
  J = J - 1;
}

void GetTimeZone(void)
{
  S = 24110.5 + 8640184.813 * T / 36525;//T0;
  S = S -(86636.6 * (float) H / 24) + 86400 * lat/ 360;//L5;
  S = S / 86400;
  S = S - Trunc(S);
  T0 = S * 360 * DR;
} 

float GetAtan()
{
  return atan(S / sqrt(1 - S * S));
}

void PosOfSun(char ind)
{
  L = 0.779072 + 0.00273790931 * T;
  G = 0.993126 + 0.0027377785 * T;
  L = L - Trunc(L);
  G = G - Trunc(G);
  L = L * P2;
  G = G * P2;
  V = 0.39785 * sin(L);
  V = V - 0.01000 * sin(L - G);
  V = V + 0.00333 * sin(L + G);
  V = V - 0.00021 * TT * sin(L);   
  U = 1 - 0.03349 * cos(G);
  U = U - 0.00014 * cos(2 * L);
  U = U + 0.00008 * cos(L);      
  S = -0.00010 - 0.04129 * sin(2 * L);
  S = S + 0.03211 * sin(G);
  S = S + 0.00104 * sin(2 * L - G);
  S = S - 0.00035 * sin(2 * L + G);
  S = S - 0.00008 * TT * sin(G);
  S = S / sqrt(U - V * V);  
  aA[ind] = L + GetAtan();
  S = V / sqrt(U);
  aD[ind] = GetAtan();
}

float getV(float vd, float vh)
{
  return (S * sin(vd) + cos(B5 * DR)* cos(vd) * cos(vh) - cos(DR * 90.833));
}

void PossibleEvents(int Hour)
{
  L = aA[0] + ((float)Hour + 1) / 24 * (aA[1] - aA[0]);
  G = aD[0] + ((float)Hour + 1) / 24 * (aD[1] - aD[0]);
  st = 0;
  A = T0 + Hour * K1 - A0;
  B = T0 + Hour * K1 + K1 - L;
  if (Hour <= 0) {V0 = getV(D0, A);}
  V2 = getV(G,B);
  if (Sgn(V0) == Sgn(V2)) {goto end2;}
  V1 = getV((G + D0) / 2,(B + A) / 2);
  A = 2 * V2 - 4 * V1 + 2 * V0;
  B = 4 * V1 - 3 * V0 - V2;
  D = B * B - 4 * A * V0;
  if (D < 0) {goto end2;};
  D = sqrt(D);
  E = (-B + D) / (2 * A);
  if ((E > 1) || (E < 0)) {E = (-B - D) / (2 * A);} 
  A = (Hour + E + 1 / 120 );
  B = Trunc((A - Trunc(A)) * 60); //M3
  if ((V0 < 0) && (V2 > 0)) {HH[0]=Trunc(A);  MM[0]=Trunc(B);  M8 = 1; st=1;};
  if ((V0 > 0) && (V2 < 0)) {HH[1]=Trunc(A);  MM[1]=Trunc(B);  W8 = 1; st=1;};
end2:   
if (st != 0) {sun_info();}
  A0 = L;
  D0 = G;
  V0 = V2;
}


void OutInform(void)
{
  if ((M8 == 0) && (W8 == 0))
 {  
    if (V2 < 0) {st=1; cld_cls();}//   
    if (V2 > 0) {st=0; cld_cls();}//   
 }
  else
  {
    if (M8 == 0) {st=1; cld_cls();} //     
    if (W8 == 0) {st=0; cld_cls();} //     
  }
}

void show_info(void)
{ 
  D = day;
  Calendar();
  T = (J - 2451545) + F;
  TT = T / 36525 + 1; // TT - ,   1900
  GetTimeZone(); //   
  T = T - (float) H / 24;
  PosOfSun(0); //   
  T = T + 1;
  PosOfSun(1); 
  if (aA[1] < aA[0]) {aA[1] = aA[1] + P2;}
  S = sin(B5 * DR);
  M8 = 0;
  W8 = 0;
  A0 = aA[0];
  D0 = aD[0];
  
  for (C0 = 0; C0 < 24; C0++)
  {PossibleEvents(C0);}
   OutInform();
}

void display(void)
{ 
  lcd_gotoxy(13,0); //13
  lcd_putsf("Z");
  lcd_putchar (H/10%10+0x30);
  lcd_putchar (H%10+0x30);
  lcd_gotoxy(0,1);
  lcd_putchar (hr/10%10+0x30);
  lcd_putchar (hr%10+0x30);
  lcd_putchar (delim);
  lcd_putchar (mins/10%10+0x30);
  lcd_putchar (mins%10+0x30);
  lcd_putsf (" ");
  lcd_putchar ((day/10)%10+0x30);
  lcd_putchar (day%10+0x30);
  lcd_putsf ("/");
  lcd_putchar ((M/10)%10+0x30);
  lcd_putchar (M%10+0x30);
  lcd_putsf ("/");
  lcd_putchar ((Y/1000)%10+0x30);
  lcd_putchar ((Y/100)%10+0x30);
  lcd_putchar ((Y/10)%10+0x30);
  lcd_putchar (Y%10+0x30);
}


void clock(void)
{ unsigned char qty_days;
  if (sec==60) {sec=0; mins++; if (delim==':') {delim=' ';} else {delim=':';}; };  //   ,    60
  if (mins==60) {mins=0; hr++;}; //   ,    60
  if (hr==24) {day++; hr=0; show_info();};//   24
  qty_days=days_month[M-1];//   
  if ((M==2) & (Y%4)==(Y/4)) {qty_days=29;}//   
  if (day>qty_days) {M++; day=1;};//   
  if (M>12) {M=1; Y++;};//   12 
  
  display();
 
 }
    
 void on_of_led(void)
  {  
    long int time_dn, time_up, time_now;//       
    time_up=(HH[0]*60 + MM[0]) - cor;// cor     
    time_dn=(HH[1]*60 + MM[1]) + cor;// cor    
    time_now=(long int)hr*60 + mins;
    if (st==1) {light_on;} else
    if ((time_now>=time_up)&&(time_now<=time_dn)) {light_off;}  //   T > T   T < T  
    else {light_on;};// ,   ,   .    ,
    //    
  }
 
void keypress(unsigned char ind, long int ddey, int ga, int gb, char sw) 
{
  while (key!=Ok)
  { 
    if (sw==1)
    {
      lcd_gotoxy(6,1);
      lcd_putchar (ddey/10%10+0x30);
      lcd_putchar (ddey%10+0x30);
    }
    else 
    {  
      display();
    } 
    if ((key==Up) & (ddey<ga)) {ddey++; delay_ms(200);}
    if ((key==Down) & (ddey>gb)) {ddey--; delay_ms(200);}
   
    if (ind==1) {B5=ddey;}   
    if (ind==2) {lat=(float)ddey;}
    if (ind==3) {M=ddey;}
    if (ind==4) {day=ddey;}
    if (ind==5) {Y=ddey;}
    if (ind==6) {hr=ddey;}  
    if (ind==7) {mins=ddey;} 
    if (ind==8) {cor=ddey;}
    if (ind==9) {H=ddey;} 
  }
  me_delay();
  lcd_clear();////////
  lcd_gotoxy(0,0);  
}

void menu(void)
{
  lcd_clear();
  lcd_putsf(msg1); 
  keypress(1, B5, 90, 0, 1); 

  lcd_putsf(msg2); //Dolg
  keypress(2, lat, 180, 0, 1); 

  lcd_putsf(msg3); 
  keypress(9, H, 12, 0, 1);

  lcd_putsf(msg4);
  keypress(8, cor, 59, 0, 1);  //

  lcd_putsf(msg5);
  keypress(3, M, 12, 1, 0); 

  lcd_putsf(msg6);
  keypress(4, day, days_month[M], 1, 0); 

  lcd_putsf(msg7);
  keypress(5, Y, 2100, 2000, 0); 

  lcd_putsf(msg8); 
  keypress(6, hr, 24, 1, 0); 

  lcd_putsf(msg9);
  keypress(7, mins, 59, 1, 0);
}


void main(void)
{

PORTB=0x00;
DDRB=0x01;
PORTC=0x1F;
DDRC=0x00;
PORTD=0x00;
DDRD=0xF7;
TCCR0=0x03;
TCNT0=0x00;
TCCR1A=0x00;
TCCR1B=0x05;
TCNT1H=0x0F;
TCNT1L=0x42;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;
ASSR=0x00;
TCCR2=0x00;
TCNT2=0x82;
OCR2=0x00;
MCUCR=0x00;
TIMSK=0x11;
ACSR=0x80;
SFIOR=0x00;


/// LCD module initialization
lcd_init(16);

#asm("sei")

lcd_gotoxy(0,0);
lcd_putsf(info);
while (key!=Ok){}
me_delay();
lcd_clear();

menu();
clock();
show_info();

while (1)
      {
      // Place your code here
          if (key !=31) 
          {
            menu();
            show_info();
          };
          clock(); 
          on_of_led();
      };
}
